@@ -2,10 +2,8 @@ from __future__ import annotations
 
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
-from functools import cached_property, lru_cache
 from itertools import islice
-from marshal import loads
-from typing import TYPE_CHECKING, Any, Iterable, Protocol, cast
+from typing import TYPE_CHECKING, Callable, Protocol
 
 import rich.repr
 from rich.console import Console, ConsoleOptions, RenderableType
@@ -13,36 +11,49 @@ from rich.measure import Measurement
 from rich.protocol import is_renderable, rich_cast
 from rich.segment import Segment
 from rich.style import Style as RichStyle
-from rich.terminal_theme import TerminalTheme
 from rich.text import Text
 
 from textual._context import active_app
-from textual.color import TRANSPARENT, Color
-from textual.css.styles import StylesBase
-from textual.css.types import AlignHorizontal, AlignVertical
+from textual.css.styles import RulesMap
 from textual.geometry import Spacing
 from textual.render import measure
+from textual.selection import Selection
 from textual.strip import Strip
+from textual.style import Style
 
 if TYPE_CHECKING:
     from typing_extensions import TypeAlias
 
     from textual.widget import Widget
 
-_NULL_RICH_STYLE = RichStyle()
-
 
 def is_visual(obj: object) -> bool:
     """Check if the given object is a Visual or supports the Visual protocol."""
     return isinstance(obj, Visual) or hasattr(obj, "textualize")
 
 
+@dataclass(frozen=True)
+class RenderOptions:
+    """Additional options passed to `Visual.render_strips`."""
+
+    get_style: Callable[[str | Style], Style]
+    """Callable to get a style."""
+    rules: RulesMap
+    """Mapping of style rules."""
+    selection: Selection | None = None
+    """Text selection information."""
+    selection_style: Style | None = None
+    """Style of text selection."""
+    post_style: Style | None = None
+    """Optional style to apply post render."""
+
+
 # Note: not runtime checkable currently, as I've found that to be slow
 class SupportsVisual(Protocol):
     """An object that supports the textualize protocol."""
 
     def visualize(self, widget: Widget, obj: object) -> Visual | None:
-        """Convert the result of a Widget.render() call in to a Visual, using the Visual protocol.
+        """Convert the result of a Widget.render() call into a Visual, using the Visual protocol.
 
         Args:
             widget: The widget that generated the render.
@@ -61,18 +72,24 @@ class VisualError(Exception):
 VisualType: TypeAlias = "RenderableType | SupportsVisual | Visual"
 
 
-def visualize(widget: Widget, obj: object) -> Visual:
+def visualize(widget: Widget, obj: object, markup: bool = True) -> Visual:
     """Get a visual instance from an object.
 
     If the object does not support the Visual protocol and is a Rich renderable, it
     will be wrapped in a [RichVisual][textual.visual.RichVisual].
 
     Args:
+        widget: The parent widget.
         obj: An object.
+        markup: Enable markup.
+
+    Raises:
+        VisualError: If there is no Visual could be returned to render `obj`.
 
     Returns:
         A Visual instance to render the object, or `None` if there is no associated visual.
     """
+    _rich_traceback_omit = True
     if isinstance(obj, Visual):
         # Already a visual
         return obj
@@ -80,10 +97,14 @@ def visualize(widget: Widget, obj: object) -> Visual:
     visualize = getattr(obj, "visualize", None)
     if visualize is None:
         # Doesn't expose the textualize protocol
+        from textual.content import Content
+
+        if isinstance(obj, str):
+            return Content.from_markup(obj) if markup else Content(obj)
+
         if is_renderable(obj):
-            # If it is a string, render it to Text
-            if isinstance(obj, str):
-                obj = widget.render_str(obj)
+            if isinstance(obj, Text):
+                return Content.from_rich_text(obj, console=widget.app.console)
 
             # If its is a Rich renderable, wrap it with a RichVisual
             return RichVisual(widget, rich_cast(obj))
@@ -99,139 +120,8 @@ def visualize(widget: Widget, obj: object) -> Visual:
     return visual
 
 
-@rich.repr.auto
-@dataclass(frozen=True)
-class Style:
-    """Represents a style in the Visual interface (color and other attributes)."""
-
-    background: Color = TRANSPARENT
-    foreground: Color = TRANSPARENT
-    bold: bool | None = None
-    dim: bool | None = None
-    italic: bool | None = None
-    underline: bool | None = None
-    strike: bool | None = None
-    link: str | None = None
-    _meta: bytes | None = None
-    auto_color: bool = False
-
-    def __rich_repr__(self) -> rich.repr.Result:
-        yield None, self.background
-        yield None, self.foreground
-        yield "bold", self.bold, None
-        yield "dim", self.dim, None
-        yield "italic", self.italic, None
-        yield "underline", self.underline, None
-        yield "strike", self.strike, None
-
-    @lru_cache(maxsize=1024)
-    def __add__(self, other: object) -> Style:
-        if not isinstance(other, Style):
-            return NotImplemented
-        new_style = Style(
-            self.background + other.background,
-            self.foreground if other.foreground.is_transparent else other.foreground,
-            self.bold if other.bold is None else other.bold,
-            self.dim if other.dim is None else other.dim,
-            self.italic if other.italic is None else other.italic,
-            self.underline if other.underline is None else other.underline,
-            self.strike if other.strike is None else other.strike,
-            self.link if other.link is None else other.link,
-            self._meta if other._meta is None else other._meta,
-        )
-        return new_style
-
-    @classmethod
-    def from_rich_style(
-        cls, rich_style: RichStyle, theme: TerminalTheme | None = None
-    ) -> Style:
-        """Build a Style from a (Rich) Style.
-
-        Args:
-            rich_style: A Rich Style object.
-            theme: Optional Rich [terminal theme][rich.terminal_theme.TerminalTheme].
-
-        Returns:
-            New Style.
-        """
-        return Style(
-            Color.from_rich_color(rich_style.bgcolor, theme),
-            Color.from_rich_color(rich_style.color, theme),
-            bold=rich_style.bold,
-            dim=rich_style.dim,
-            italic=rich_style.italic,
-            underline=rich_style.underline,
-            strike=rich_style.strike,
-        )
-
-    @classmethod
-    def from_styles(cls, styles: StylesBase) -> Style:
-        """Create a Visual Style from a Textual styles object.
-
-        Args:
-            styles: A Styles object, such as `my_widget.styles`.
-
-        """
-        text_style = styles.text_style
-        return Style(
-            styles.background,
-            (
-                Color(0, 0, 0, styles.color.a, auto=True)
-                if styles.auto_color
-                else styles.color
-            ),
-            bold=text_style.bold,
-            dim=text_style.italic,
-            italic=text_style.italic,
-            underline=text_style.underline,
-            strike=text_style.strike,
-            auto_color=styles.auto_color,
-        )
-
-    @cached_property
-    def rich_style(self) -> RichStyle:
-        """Convert this Styles in to a Rich style.
-
-        Returns:
-            A Rich style object.
-        """
-        return RichStyle(
-            color=(self.background + self.foreground).rich_color,
-            bgcolor=self.background.rich_color,
-            bold=self.bold,
-            dim=self.dim,
-            italic=self.italic,
-            underline=self.underline,
-            strike=self.strike,
-            link=self.link,
-            meta=self.meta,
-        )
-
-    @cached_property
-    def without_color(self) -> Style:
-        return Style(
-            bold=self.bold,
-            dim=self.dim,
-            italic=self.italic,
-            strike=self.strike,
-            link=self.link,
-            _meta=self._meta,
-        )
-
-    @classmethod
-    def combine(cls, styles: Iterable[Style]) -> Style:
-        """Add a number of styles and get the result."""
-        iter_styles = iter(styles)
-        return sum(iter_styles, next(iter_styles))
-
-    @property
-    def meta(self) -> dict[str, Any]:
-        """Get meta information (can not be changed after construction)."""
-        return {} if self._meta is None else cast(dict[str, Any], loads(self._meta))
-
-
 class Visual(ABC):
-    """A Textual 'visual' object.
+    """A Textual 'Visual' object.
 
     Analogous to a Rich renderable, but with support for transparency.
 
@@ -239,35 +129,59 @@ class Visual(ABC):
 
     @abstractmethod
     def render_strips(
-        self, widget: Widget, width: int, height: int | None, style: Style
+        self, width: int, height: int | None, style: Style, options: RenderOptions
     ) -> list[Strip]:
-        """Render the visual in to an iterable of strips.
+        """Render the Visual into an iterable of strips.
 
         Args:
-            base_style: The base style.
             width: Width of desired render.
             height: Height of desired render or `None` for any height.
-            style: A Visual Style.
+            style: The base style to render on top of.
+            options: Additional render options.
 
         Returns:
             An list of Strips.
         """
 
     @abstractmethod
-    def get_optimal_width(self, container_width: int) -> int:
-        """Get ideal width of the renderable to display its content.
+    def get_optimal_width(self, rules: RulesMap, container_width: int) -> int:
+        """Get optimal width of the Visual to display its content.
+
+        The exact definition of "optimal width" is dependant on the Visual, but
+        will typically be wide enough to display output without cropping or wrapping,
+        and without superfluous space.
 
         Args:
-            container_size: The size of the container.
+            rules: A mapping of style rules, such as the Widgets `styles` object.
+            container_width: The width of the container, used by Rich Renderables.
+                May be ignored for Textual Visuals.
 
         Returns:
             A width in cells.
 
         """
 
+    def get_minimal_width(self, rules: RulesMap) -> int:
+        """Get a minimal width (the smallest width before data loss occurs).
+
+        Args:
+            rules: A mapping of style rules, such as the Widgets `styles` object.
+            container_width: The width of the container, used by Rich Renderables.
+                May be ignored for Textual Visuals.
+
+        Returns:
+            A width in cells.
+
+        """
+        return 1
+
     @abstractmethod
-    def get_height(self, width: int) -> int:
-        """Get the height of the visual if rendered with the given width.
+    def get_height(self, rules: RulesMap, width: int) -> int:
+        """Get the height of the Visual if rendered at the given width.
+
+        Args:
+            rules: A mapping of style rules, such as the Widgets `styles` object.
+            width: Width of visual in cells.
 
         Returns:
             A height in lines.
@@ -283,7 +197,7 @@ class Visual(ABC):
         style: Style,
         *,
         pad: bool = False,
-        align: tuple[AlignHorizontal, AlignVertical] = ("left", "top"),
+        post_style: Style | None = None,
     ) -> list[Strip]:
         """High level function to render a visual to strips.
 
@@ -294,19 +208,41 @@ class Visual(ABC):
             height: Desired height (in lines) or `None` for no limit.
             style: A (Visual) Style instance.
             pad: Pad to desired width?
-            align: Tuple of horizontal and vertical alignment.
+            post_style: Optional Style to apply to strips after rendering.
 
         Returns:
             A list of Strips containing the render.
         """
-        strips = visual.render_strips(widget, width, height, style)
+
+        selection = widget.text_selection
+        if selection is not None:
+            selection_style: Style | None = Style.from_rich_style(
+                widget.screen.get_component_rich_style("screen--selection")
+            )
+        else:
+            selection_style = None
+
+        strips = visual.render_strips(
+            width,
+            height,
+            style,
+            RenderOptions(
+                widget._get_style,
+                widget.styles,
+                selection,
+                selection_style,
+            ),
+        )
+        strips = [strip._apply_link_style(widget.link_style) for strip in strips]
+
         if height is None:
             height = len(strips)
-        rich_style = style.rich_style
+        rich_style = (style + Style(reverse=False)).rich_style
         if pad:
             strips = [strip.extend_cell_length(width, rich_style) for strip in strips]
-        if align != ("left", "top"):
-            align_horizontal, align_vertical = align
+        content_align = widget.styles.content_align
+        if content_align != ("left", "top"):
+            align_horizontal, align_vertical = content_align
             strips = list(
                 Strip.align(
                     strips,
@@ -349,16 +285,16 @@ class RichVisual(Visual):
             )
         return self._measurement
 
-    def get_optimal_width(self, container_width: int) -> int:
+    def get_optimal_width(self, rules: RulesMap, container_width: int) -> int:
         console = active_app.get().console
         width = measure(
             console, self._renderable, container_width, container_width=container_width
         )
-
         return width
 
-    def get_height(self, width: int) -> int:
-        console = active_app.get().console
+    def get_height(self, rules: RulesMap, width: int) -> int:
+        app = active_app.get()
+        console = app.console
         renderable = self._renderable
         if isinstance(renderable, Text):
             height = len(
@@ -370,7 +306,8 @@ class RichVisual(Visual):
                 )
             )
         else:
-            options = console.options.update_width(width).update(highlight=False)
+            console_options = app.console_options
+            options = console_options.update_width(width).update(highlight=False)
             segments = console.render(renderable, options)
             # Cheaper than counting the lines returned from render_lines!
             height = sum([text.count("\n") for text, _, _ in segments])
@@ -378,21 +315,29 @@ class RichVisual(Visual):
         return height
 
     def render_strips(
-        self,
-        widget: Widget,
-        width: int,
-        height: int | None,
-        style: Style,
+        self, width: int, height: int | None, style: Style, options: RenderOptions
     ) -> list[Strip]:
-        console = active_app.get().console
-        options = console.options.update(
+        """Render the Visual into an iterable of strips. Part of the Visual protocol.
+
+        Args:
+            width: Width of desired render.
+            height: Height of desired render or `None` for any height.
+            style: The base style to render on top of.
+            options: Additional render options.
+
+        Returns:
+            An list of Strips.
+        """
+        app = active_app.get()
+        console = app.console
+        console_options = app.console_options.update(
             highlight=False,
             width=width,
             height=height,
         )
         rich_style = style.rich_style
-        renderable = widget.post_render(self._renderable, rich_style)
-        segments = console.render(renderable, options.update_width(width))
+        renderable = self._widget.post_render(self._renderable, rich_style)
+        segments = console.render(renderable, console_options.update_width(width))
         strips = [
             Strip(line)
             for line in islice(
@@ -403,6 +348,7 @@ class RichVisual(Visual):
                 height,
             )
         ]
+
         return strips
 
 
@@ -410,7 +356,7 @@ class RichVisual(Visual):
 class Padding(Visual):
     """A Visual to pad another visual."""
 
-    def __init__(self, visual: Visual, spacing: Spacing):
+    def __init__(self, visual: Visual, spacing: Spacing) -> None:
         """
 
         Args:
@@ -424,29 +370,42 @@ class Padding(Visual):
         yield self._visual
         yield self._spacing
 
-    def get_optimal_width(self, container_width: int) -> int:
-        return self._visual.get_optimal_width(container_width) + self._spacing.width
+    def get_optimal_width(self, rules: RulesMap, container_width: int) -> int:
+        return (
+            self._visual.get_optimal_width(rules, container_width) + self._spacing.width
+        )
 
-    def get_height(self, width: int) -> int:
-        return self._visual.get_height(width) + self._spacing.height
+    def get_height(self, rules: RulesMap, width: int) -> int:
+        return (
+            self._visual.get_height(rules, width - self._spacing.width)
+            + self._spacing.height
+        )
 
     def render_strips(
-        self,
-        widget: Widget,
-        width: int,
-        height: int | None,
-        style: Style,
+        self, width: int, height: int | None, style: Style, options: RenderOptions
     ) -> list[Strip]:
+        """Render the Visual into an iterable of strips. Part of the Visual protocol.
+
+        Args:
+            width: Width of desired render.
+            height: Height of desired render or `None` for any height.
+            style: The base style to render on top of.
+            options: Additional render options.
+
+        Returns:
+            An list of Strips.
+        """
         padding = self._spacing
         top, right, bottom, left = self._spacing
         render_width = width - (left + right)
         if render_width <= 0:
             return []
+
         strips = self._visual.render_strips(
-            widget,
             render_width,
             None if height is None else height - padding.height,
             style,
+            options,
         )
 
         if padding:
